@@ -10,7 +10,7 @@
 #define SHELL_Q '"'
 #define DL_DFLT_NAME "libpari.dll"
 
-#define PARIVERSION "GP/PARI CALCULATOR Version 2.3.3 (released)"
+#define PARIVERSION "GP/PARI CALCULATOR Version 2.3.4 (released)"
 #ifdef __cplusplus
 #  define PARIINFO_PRE "C++"
 #else
@@ -26,7 +26,7 @@
 #  define PARIINFO_POST "ix86 running Windows 3.2 (ix86 kernel) 32-bit version"
 #endif
 #define PARIINFO PARIINFO_PRE/**/PARIINFO_POST
-#define PARI_VERSION_CODE 131843
+#define PARI_VERSION_CODE 131844
 #define PARI_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
 #define PARI_VERSION_SHIFT 8
 
@@ -61,9 +61,6 @@
 /* No exp2, log2 in libc */
 #define NOEXP2
 
-/* Headers are clean - ulong not defined */
-#define ULONG_NOT_DEFINED
-
 #if !defined(WINCE) && !(defined(_WIN32) && defined(_MSC_VER))
 /* Timings: Don't use times because of the HZ / CLK_TCK confusion. */
 #  define USE_FTIME 1
