@@ -1,4 +1,4 @@
-/* $Id: alglin2.c,v 1.223.2.1 2006/10/03 23:15:33 kb Exp $
+/* $Id: alglin2.c 9661 2008-02-18 10:07:27Z kb $
 
 Copyright (C) 2000  The PARI group.
 
@@ -3082,11 +3082,9 @@ hnfall_i(GEN A, GEN *ptB, long remove)
     for (i=h[j]; i; i--)
     {
       a = gcoeff(A,i,j);
-      if (!signe(a)) continue;
-
       k = c[i];
-      ZV_elem(a,gcoeff(A,i,k), A,B, j,k);
-      ZM_reduce(A,B, i,k);
+      if (signe(a)) ZV_elem(a,gcoeff(A,i,k), A,B, j,k);
+      ZM_reduce(A,B, i,k); /* ensure non-negative entries, even if a = 0 */
       if (low_stack(lim, stack_lim(av1,1)))
       {
         if (DEBUGMEM>1) pari_warn(warnmem,"hnfall[3], j = %ld", j);
