@@ -1,5 +1,5 @@
 #line 2 "../src/kernel/none/mp_indep.c"
-/* $Id: mp_indep.c,v 1.33.2.1 2006/05/31 08:30:49 kb Exp $
+/* $Id: mp_indep.c 10294 2008-06-10 15:58:51Z kb $
 
 Copyright (C) 2000  The PARI group.
 
@@ -291,8 +291,8 @@ if (!gequal(U, z)) pari_err(talker,"toto");
   }
   if (garde & HIGHBIT)
   { /* round to nearest */
-    i = lz; do z[--i]++; while (z[i]==0 && i > 1);
-    if (i == 1) { z[2] = HIGHBIT; ez++; }
+    i = lz; do z[--i]++; while (z[i]==0 && i > 2);
+    if (z[i] == 0) { z[2] = (long)HIGHBIT; ez++; }
   }
   z[1] = evalsigne(sz) | evalexpo(ez);
 }
