@@ -1,4 +1,4 @@
-/* $Id: nffactor.c,v 1.207.2.1 2006/08/03 16:36:51 kb Exp $
+/* $Id: nffactor.c 10291 2008-06-10 15:50:00Z kb $
 
 Copyright (C) 2000-2004  The PARI group.
 
@@ -135,6 +135,52 @@ QXQX_normalize(GEN P, GEN T)
   return P;
 }
 
+/* to INT / FRAC / (POLMOD mod T), not memory clean because T not copied */
+static GEN
+RgXQ_to_mod(GEN x, GEN T)
+{
+  long d;
+  switch(typ(x))
+  {
+    case t_INT: case t_FRAC:
+      return gcopy(x);
+
+    default:
+      d = degpol(x);
+      if (d < 0) return gen_0;
+      if (d == 0) return gcopy(gel(x,2));
+      return mkpolmod(gcopy(x), T);
+  }
+}
+/* T a ZX, z lifted from (Q[Y]/(T(Y)))[X], apply RgXQ_to_mod to all coeffs.
+ * Not memory clean because T not copied */
+static GEN
+RgXQX_to_mod(GEN z, GEN T)
+{
+  long i,l = lg(z);
+  GEN x = cgetg(l,t_POL);
+  for (i=2; i<l; i++) gel(x,i) = RgXQ_to_mod(gel(z,i), T);
+  x[1] = z[1]; return normalizepol_i(x,l);
+}
+/* Apply RgXQX_to_mod to all entries. Memory-clean ! */
+static GEN
+RgXQXV_to_mod(GEN V, GEN T)
+{
+  long i, l = lg(V);
+  GEN z = cgetg(l, t_VEC); T = gcopy(T);
+  for (i=1;i<l; i++) gel(z,i) = RgXQX_to_mod(gel(V,i), T);
+  return z;
+}
+/* Apply RgXQ_to_mod to all entries. Memory-clean ! */
+static GEN
+RgXQV_to_mod(GEN V, GEN T)
+{
+  long i, l = lg(V);
+  GEN z = cgetg(l, t_VEC); T = gcopy(T);
+  for (i=1;i<l; i++) gel(z,i) = RgXQ_to_mod(gel(V,i), T);
+  return z;
+}
+
 /* return the roots of pol in nf */
 GEN
 nfroots(GEN nf,GEN pol)
@@ -169,6 +215,7 @@ nfroots(GEN nf,GEN pol)
   A = QXQX_normalize(A, T);
   A = Q_primpart(A);
   A = nfsqff(nf,A,1);
+  A = RgXQV_to_mod(A, T);
   return gerepileupto(av, gen_sort(A, 0, cmp_pol));
 }
 
@@ -243,13 +290,21 @@ nf_pol_lift(GEN pol, GEN bound, nfcmbf_t *T)
   return x;
 }
 
+static GEN
+zerofact(long v)
+{
+  GEN z = cgetg(3, t_MAT);
+  gel(z,1) = mkcol(zeropol(v));
+  gel(z,2) = mkcol(gen_1); return z;
+}
+
 /* return the factorization of x in nf */
 GEN
 nffactor(GEN nf,GEN pol)
 {
-  GEN A,g,y,p1,rep,T;
-  long l, j, d = degpol(pol);
-  pari_sp av;
+  GEN A,g,y,p1,T, rep = cgetg(3, t_MAT);
+  long l, j, dA;
+  pari_sp av = avma;
   pari_timer ti;
 
   if (DEBUGLEVEL>2) { TIMERstart(&ti); fprintferr("\nEntering nffactor:\n"); }
@@ -258,18 +313,22 @@ nffactor(GEN nf,GEN pol)
   if (varncmp(varn(pol), varn(T)) >= 0)
     pari_err(talker,"polynomial variable must have highest priority in nffactor");
 
-  if (d == 0) return trivfact();
-  rep = cgetg(3, t_MAT); av = avma;
-  if (d == 1)
-  {
-    gel(rep,1) = mkcol( gcopy(pol) );
-    gel(rep,2) = mkcol( gen_1 );
-    return rep;
-  }
-
   A = fix_relative_pol(nf,pol,0);
-  if (degpol(nf[1]) == 1)
-    return gerepileupto(av, factpol(simplify(pol), 0));
+  dA = degpol(A);
+  if (dA <= 0) {
+    avma = (pari_sp)(rep + 3);
+    return dA == 0? trivfact(): zerofact(varn(pol));
+  }
+  A = Q_primpart( QXQX_normalize(A, T) );
+  if (dA == 1) {
+    GEN c;
+    A = gerepilecopy(av, A); c = gel(A,2);
+    if (typ(c) == t_POL && degpol(c) > 0) gel(A,2) = mkpolmod(c, gcopy(T));
+    gel(rep,1) = mkcol(A);
+    gel(rep,2) = mkcol(gen_1); return rep;
+  }
+  if (degpol(T) == 1)
+    return gerepileupto(av, factpol(Q_primpart(simplify(pol)), 0));
 
   A = Q_primpart( lift_intern(A) );
   g = nfgcd(A, derivpol(A), T, gel(nf,4));
@@ -300,13 +359,13 @@ nffactor(GEN nf,GEN pol)
       }
       ex[j] = e;
     }
-    avma = av1; y = gerepileupto(av, y);
+    avma = av1; y = gerepileupto(av, RgXQXV_to_mod(y,T));
     p1 = cgetg(l, t_COL); for (j=l-1; j>=1; j--) gel(p1,j) = utoipos(ex[j]);
     free(ex);
   }
   else
   {
-    y = gerepileupto(av, nfsqff(nf,A,0));
+    y = gerepileupto(av, RgXQXV_to_mod(nfsqff(nf,A,0), T));
     l = lg(y);
     p1 = cgetg(l, t_COL); for (j=l-1; j>=1; j--) gel(p1,j) = gen_1;
   }
@@ -1267,8 +1326,8 @@ AGAIN:
 static GEN
 nf_combine_factors(nfcmbf_t *T, GEN polred, GEN p, long a, long klim)
 {
-  GEN z, res, L, listmod, famod = T->fact, nf = T->nf;
-  long i, m, l, maxK = 3, nft = lg(famod)-1;
+  GEN res, L, listmod, famod = T->fact, nf = T->nf;
+  long l, maxK = 3, nft = lg(famod)-1;
   pari_timer ti;
 
   if (DEBUGLEVEL>2) TIMERstart(&ti);
@@ -1284,15 +1343,16 @@ nf_combine_factors(nfcmbf_t *T, GEN polred, GEN p, long a, long klim)
   famod = gel(listmod,l);
   if (maxK >= 0 && lg(famod)-1 > 2*maxK)
   {
-    if (l > 1) T->polbase = unifpol(nf, gel(res,l), t_COL);
+    if (l > 1)
+    {
+      T->pol = gel(res,l);
+      T->polbase = unifpol(nf, gel(res,l), t_COL);
+    }
     L = nf_LLL_cmbf(T, p, a, maxK);
     /* remove last elt, possibly unfactored. Add all new ones. */
     setlg(res, l); res = shallowconcat(res, L);
   }
-
-  m = lg(res); z = cgetg(m, t_VEC);
-  for (i=1;i<m; i++) gel(z,i) = unifpol(nf,gel(res,i), t_POLMOD);
-  return z;
+  return res;
 }
 
 static GEN
@@ -1446,7 +1506,6 @@ static GEN
 nfsqff(GEN nf, GEN pol, long fl)
 {
   long n, nbf, dpol = degpol(pol);
-  pari_sp av = avma;
   GEN pr, C0, polbase, init_fa = NULL;
   GEN N2, rep, polmod, polred, lt, nfpol = gel(nf,1);
   nfcmbf_t T;
@@ -1457,9 +1516,8 @@ nfsqff(GEN nf, GEN pol, long fl)
   n = degpol(nfpol);
   polbase = unifpol(nf, pol, t_COL);
   if (typ(polbase) != t_POL) pari_err(typeer, "nfsqff");
-  polmod  = unifpol(nf, pol, t_POLMOD);
-  if (dpol == 1) /* irreducible */
-    return gerepilecopy(av, mkvec(QXQX_normalize(polmod, nfpol)));
+  polmod  = lift_intern( unifpol(nf, pol, t_POLMOD) );
+  if (dpol == 1) return mkvec(QXQX_normalize(polmod, nfpol));
   /* heuristic */
   if (dpol*3 < n) 
   {
@@ -1475,17 +1533,16 @@ nfsqff(GEN nf, GEN pol, long fl)
         gel(z,i) = gneg(gdiv(gel(t,2), gel(t,3)));
       }
       setlg(z, i);
-      if (fl == 2 && i != l) { avma = av; return cgetg(1,t_VEC); }
+      if (fl == 2 && i != l) return cgetg(1,t_VEC);
     }
-    return gerepilecopy(av, z);
+    return z;
   }
 
   nbf = nf_pick_prime(5, nf, polbase, fl, &lt, &init_fa, &pr, &L.Tp);
   if (fl == 2 && nbf < dpol) return cgetg(1,t_VEC);
   if (nbf <= 1)
   {
-    if (!fl) /* irreducible */
-      return gerepilecopy(av, mkvec(QXQX_normalize(polmod, nfpol)));
+    if (!fl) return mkvec(QXQX_normalize(polmod, nfpol)); /* irreducible */
     if (!nbf) return cgetg(1,t_VEC); /* no root */
   }
 
@@ -1522,22 +1579,22 @@ nfsqff(GEN nf, GEN pol, long fl)
 
   if (fl) {
     GEN z = nf_DDF_roots(pol, polred, nfpol, lt, init_fa, nbf, fl, &L);
-    if (lg(z) == 1) { avma = av; return cgetg(1, t_VEC); }
-    return gerepilecopy(av,z);
+    if (lg(z) == 1) return cgetg(1, t_VEC);
+    return z;
   }
 
   {
-    pari_sp av2 = avma;
+    pari_sp av = avma;
     if (L.Tp)
       rep = FqX_split_all(init_fa, L.Tp, L.p);
     else
     {
       long d;
-      rep = cgetg(dpol + 1, t_VEC); gel(rep,1) = polred;
+      rep = cgetg(dpol + 1, t_VEC); gel(rep,1) = FpX_red(polred,L.p);
       d = FpX_split_Berlekamp((GEN*)(rep + 1), L.p);
       setlg(rep, d + 1);
     }
-    T.fact  = gerepilecopy(av2, sort_vecpol(rep, &cmp_pol));
+    T.fact  = gerepilecopy(av, sort_vecpol(rep, &cmp_pol));
   }
   if (DEBUGLEVEL>2) msgTIMER(&ti, "splitting mod %Z", pr);
   T.pr = pr;
@@ -1550,13 +1607,15 @@ nfsqff(GEN nf, GEN pol, long fl)
   rep = nf_combine_factors(&T, polred, L.p, L.k, dpol-1);
   if (DEBUGLEVEL>2)
     fprintferr("Total Time: %ld\n===========\n", TIMER(&ti_tot));
-  return gerepileupto(av, rep);
+  return rep;
 }
 
 GEN
 nfrootsall_and_pr(GEN nf, GEN pol)
 {
-  GEN J1,J2, pr, T, z = nfsqff(checknf(nf), pol, 2);
+  GEN J1,J2, pr, T;
+  pari_sp av = avma;
+  GEN z = gerepileupto(av, nfsqff(checknf(nf), pol, 2));
   if (lg(z) == 1) return NULL;
   (void)nf_pick_prime(1, nf, unifpol(nf, pol, t_COL), 2,
                       &J1, &J2, &pr, &T);
