@@ -1,4 +1,4 @@
-/* $Id: thue.c,v 1.76.2.2 2007-10-29 10:35:15 kb Exp $
+/* $Id: thue.c 10292 2008-06-10 15:55:18Z kb $
 
 Copyright (C) 2000  The PARI group.
 
@@ -967,7 +967,8 @@ thue(GEN tnf, GEN rhs, GEN ne)
     GEN c0   = gel(tnf,2); /* t_REAL */
     S = cgetg(1,t_VEC);
     ro = roots(P, DEFAULTPREC);
-    x3 = sqrtnr(mulir(constant_term(P), divir(absi(rhs),c0)), degpol(P));
+    x3 = sqrtnr(mulir(absi(rhs),c0), degpol(P));
+    x3 = addrr(x3, dbltor(0.1)); /* guard from round-off errors */
   }
 
   if (DEBUGLEVEL>=2)
@@ -1246,7 +1247,7 @@ bnfisintnorm(GEN bnf, GEN a)
   for (i=1; i<l; i++)
   {
     x = gel(res,i);
-    sNx = signe(subres(x, gel(nf,1)));
+    sNx = signe(subres(gel(nf,1), x));
     if (sNx != sa)
     {
       if (! unit) norm_1 = get_unit_1(bnf, &unit);
